import base64
from typing import Any, Optional, Union

from ...api.api_calls.get_async_operation import asyncio as fc_asyncio, sync as fc_sync
from ...client import Client
from ...models import Error
from ...models.file_conversion import FileConversion


def sync(
    id: str,
    *,
    client: Client,
) -> Optional[Union[Any, FileConversion, Error]]:
    """Get the status of a file conversion. This function automatically base64 decodes the output response if there is one."""

    fc = fc_sync(
        id=id,
        client=client,
    )

    if isinstance(fc, FileConversion) and fc.output != "":
        if isinstance(fc.output, str):
            b = base64.b64decode(fc.output)
            # decode the bytes to a string
            fc.output = b.decode("utf-8")

    return fc


async def asyncio(
    id: str,
    *,
    client: Client,
) -> Optional[Union[Any, FileConversion, Error]]:
    """Get the status of a file conversion. This function automatically base64 decodes the output response if there is one."""

    fc = await fc_asyncio(
        id=id,
        client=client,
    )

    if isinstance(fc, FileConversion) and fc.output != "":
        if isinstance(fc.output, str):
            b = base64.b64decode(fc.output)
            # decode the bytes to a string
            fc.output = b.decode("utf-8")

    return fc



from enum import Enum


class UnitMass(str, Enum):
    """The valid types of mass units."""  # noqa: E501

    """# Carats <https://en.wikipedia.org/wiki/Carat_(mass)> """  # noqa: E501
    CARATS = "carats"
    """# Grains <https://en.wikipedia.org/wiki/Grain_(unit)> """  # noqa: E501
    GRAINS = "grains"
    """# Grams <https://en.wikipedia.org/wiki/Gram> """  # noqa: E501
    GRAMS = "grams"
    """# Kilograms <https://en.wikipedia.org/wiki/Kilogram> """  # noqa: E501
    KILOGRAMS = "kilograms"
    """# Long tons <https://en.wikipedia.org/wiki/Long_ton> """  # noqa: E501
    LONG_TONS = "long_tons"
    """# Metric tons <https://en.wikipedia.org/wiki/Tonne> """  # noqa: E501
    METRIC_TONS = "metric_tons"
    """# Micrograms <https://en.wikipedia.org/wiki/Microgram> """  # noqa: E501
    MICROGRAMS = "micrograms"
    """# Milligrams <https://en.wikipedia.org/wiki/Milligram> """  # noqa: E501
    MILLIGRAMS = "milligrams"
    """# Ounces <https://en.wikipedia.org/wiki/Ounce> """  # noqa: E501
    OUNCES = "ounces"
    """# Pennyweights <https://en.wikipedia.org/wiki/Pennyweight> """  # noqa: E501
    PENNYWEIGHTS = "pennyweights"
    """# Pounds <https://en.wikipedia.org/wiki/Pound_(mass)> """  # noqa: E501
    POUNDS = "pounds"
    """# Short tons <https://en.wikipedia.org/wiki/Short_ton> """  # noqa: E501
    SHORT_TONS = "short_tons"
    """# Stones <https://en.wikipedia.org/wiki/Stone_(unit)> """  # noqa: E501
    STONES = "stones"
    """# Troy ounces <https://en.wikipedia.org/wiki/Troy_ounce> """  # noqa: E501
    TROY_OUNCES = "troy_ounces"
    """# Troy pounds <https://en.wikipedia.org/wiki/Troy_pound> """  # noqa: E501
    TROY_POUNDS = "troy_pounds"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class SystemInfoIsolationEnum(str, Enum):
    EMPTY = ""
    DEFAULT = "default"
    HYPERV = "hyperv"
    PROCESS = "process"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class Environment(str, Enum):
    """The environment the server is running in."""  # noqa: E501

    """# The development environment. This is for running locally. """  # noqa: E501
    DEVELOPMENT = "DEVELOPMENT"
    """# The preview environment. This is when PRs are created and a service is deployed for testing. """  # noqa: E501
    PREVIEW = "PREVIEW"
    """# The production environment. """  # noqa: E501
    PRODUCTION = "PRODUCTION"

    def __str__(self) -> str:
        return str(self.value)



import base64
from typing import Any, Optional, Union

from ...api.file.create_file_conversion import asyncio as fc_asyncio, sync as fc_sync
from ...client import Client
from ...models import Error, FileConversion, FileExportFormat, FileImportFormat


def sync(
    src_format: FileImportFormat,
    output_format: FileExportFormat,
    body: bytes,
    *,
    client: Client,
) -> Optional[Union[Any, FileConversion, Error]]:
    """Convert a CAD file from one format to another. If the file being converted is larger than a certain size it will be performed asynchronously. This function automatically base64 encodes the request body and base64 decodes the request output."""

    encoded = base64.b64encode(body)

    fc = fc_sync(
        src_format=src_format,
        output_format=output_format,
        body=encoded,
        client=client,
    )

    if isinstance(fc, FileConversion) and fc.output != "":
        if isinstance(fc.output, str):
            b = base64.b64decode(fc.output)
            # decode the bytes to a string
            fc.output = b.decode("utf-8")

    return fc


async def asyncio(
    src_format: FileImportFormat,
    output_format: FileExportFormat,
    body: bytes,
    *,
    client: Client,
) -> Optional[Union[Any, FileConversion, Error]]:
    """Convert a CAD file from one format to another. If the file being converted is larger than a certain size it will be performed asynchronously. This function automatically base64 encodes the request body and base64 decodes the request output."""

    encoded = base64.b64encode(body)

    fc = await fc_asyncio(
        src_format=src_format,
        output_format=output_format,
        body=encoded,
        client=client,
    )

    if isinstance(fc, FileConversion) and fc.output != "":
        if isinstance(fc.output, str):
            b = base64.b64decode(fc.output)
            # decode the bytes to a string
            fc.output = b.decode("utf-8")

    return fc



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

Y = TypeVar("Y", bound="Gateway")


@attr.s(auto_attribs=True)
class Gateway:
    """Gateway information."""  # noqa: E501

    auth_timeout: Union[Unset, int] = UNSET
    host: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    port: Union[Unset, int] = UNSET
    tls_timeout: Union[Unset, int] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        auth_timeout = self.auth_timeout
        host = self.host
        name = self.name
        port = self.port
        tls_timeout = self.tls_timeout

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if auth_timeout is not UNSET:
            field_dict["auth_timeout"] = auth_timeout
        if host is not UNSET:
            field_dict["host"] = host
        if name is not UNSET:
            field_dict["name"] = name
        if port is not UNSET:
            field_dict["port"] = port
        if tls_timeout is not UNSET:
            field_dict["tls_timeout"] = tls_timeout

        return field_dict

    @classmethod
    def from_dict(cls: Type[Y], src_dict: Dict[str, Any]) -> Y:
        d = src_dict.copy()
        auth_timeout = d.pop("auth_timeout", UNSET)

        host = d.pop("host", UNSET)

        name = d.pop("name", UNSET)

        port = d.pop("port", UNSET)

        tls_timeout = d.pop("tls_timeout", UNSET)

        gateway = cls(
            auth_timeout=auth_timeout,
            host=host,
            name=name,
            port=port,
            tls_timeout=tls_timeout,
        )

        gateway.additional_properties = d
        return gateway

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.system import System
from ..types import UNSET, Unset

K = TypeVar("K", bound="Gltf")


@attr.s(auto_attribs=True)
class Gltf:
    """Binary glTF 2.0. We refer to this as glTF since that is how our customers refer to it, but this can also import binary glTF (glb)."""  # noqa: E501

    type: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[K], src_dict: Dict[str, Any]) -> K:
        d = src_dict.copy()
        type = d.pop("type", UNSET)

        gltf = cls(
            type=type,
        )

        gltf.additional_properties = d
        return gltf

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


V = TypeVar("V", bound="Step")


@attr.s(auto_attribs=True)
class Step:
    """ISO 10303-21 (STEP) format."""  # noqa: E501

    coords: Union[Unset, System] = UNSET
    type: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.coords, Unset):
            coords = self.coords
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if coords is not UNSET:
            field_dict["coords"] = coords
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[V], src_dict: Dict[str, Any]) -> V:
        d = src_dict.copy()
        _coords = d.pop("coords", UNSET)
        coords: Union[Unset, System]
        if isinstance(_coords, Unset):
            coords = UNSET
        else:
            coords = System(_coords)

        type = d.pop("type", UNSET)

        step = cls(
            coords=coords,
            type=type,
        )

        step.additional_properties = d
        return step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


R = TypeVar("R", bound="Obj")


@attr.s(auto_attribs=True)
class Obj:
    """Wavefront OBJ format."""  # noqa: E501

    coords: Union[Unset, System] = UNSET
    type: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.coords, Unset):
            coords = self.coords
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if coords is not UNSET:
            field_dict["coords"] = coords
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[R], src_dict: Dict[str, Any]) -> R:
        d = src_dict.copy()
        _coords = d.pop("coords", UNSET)
        coords: Union[Unset, System]
        if isinstance(_coords, Unset):
            coords = UNSET
        else:
            coords = System(_coords)

        type = d.pop("type", UNSET)

        obj = cls(
            coords=coords,
            type=type,
        )

        obj.additional_properties = d
        return obj

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


N = TypeVar("N", bound="Stl")


@attr.s(auto_attribs=True)
class Stl:
    """*ST**ereo**L**ithography format."""  # noqa: E501

    coords: Union[Unset, System] = UNSET
    type: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.coords, Unset):
            coords = self.coords
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if coords is not UNSET:
            field_dict["coords"] = coords
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[N], src_dict: Dict[str, Any]) -> N:
        d = src_dict.copy()
        _coords = d.pop("coords", UNSET)
        coords: Union[Unset, System]
        if isinstance(_coords, Unset):
            coords = UNSET
        else:
            coords = System(_coords)

        type = d.pop("type", UNSET)

        stl = cls(
            coords=coords,
            type=type,
        )

        stl.additional_properties = d
        return stl

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


InputFormat = Union[Gltf, Step, Obj, Stl]



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.modeling_cmd import ModelingCmd
from ..models.modeling_cmd_id import ModelingCmdId
from ..types import UNSET, Unset

C = TypeVar("C", bound="ModelingCmdReq")


@attr.s(auto_attribs=True)
class ModelingCmdReq:
    """A graphics command submitted to the KittyCAD engine via the Modeling API."""  # noqa: E501

    cmd: Union[Unset, ModelingCmd] = UNSET
    cmd_id: Union[Unset, ModelingCmdId] = UNSET
    file_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.cmd, Unset):
            cmd = self.cmd
        if not isinstance(self.cmd_id, Unset):
            cmd_id = self.cmd_id
        file_id = self.file_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cmd is not UNSET:
            field_dict["cmd"] = cmd
        if cmd_id is not UNSET:
            field_dict["cmd_id"] = cmd_id
        if file_id is not UNSET:
            field_dict["file_id"] = file_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[C], src_dict: Dict[str, Any]) -> C:
        d = src_dict.copy()
        _cmd = d.pop("cmd", UNSET)
        cmd: Union[Unset, ModelingCmd]
        if isinstance(_cmd, Unset):
            cmd = UNSET
        else:
            cmd = _cmd  # type: ignore[arg-type]

        _cmd_id = d.pop("cmd_id", UNSET)
        cmd_id: Union[Unset, ModelingCmdId]
        if isinstance(_cmd_id, Unset):
            cmd_id = UNSET
        else:
            cmd_id = ModelingCmdId(_cmd_id)

        file_id = d.pop("file_id", UNSET)

        modeling_cmd_req = cls(
            cmd=cmd,
            cmd_id=cmd_id,
            file_id=file_id,
        )

        modeling_cmd_req.additional_properties = d
        return modeling_cmd_req

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .account_provider import AccountProvider
from .ai_plugin_api import AiPluginApi
from .ai_plugin_api_type import AiPluginApiType
from .ai_plugin_auth import AiPluginAuth
from .ai_plugin_auth_type import AiPluginAuthType
from .ai_plugin_http_auth_type import AiPluginHttpAuthType
from .ai_plugin_manifest import AiPluginManifest
from .api_call_query_group import ApiCallQueryGroup
from .api_call_query_group_by import ApiCallQueryGroupBy
from .api_call_status import ApiCallStatus
from .api_call_with_price import ApiCallWithPrice
from .api_call_with_price_results_page import ApiCallWithPriceResultsPage
from .api_token import ApiToken
from .api_token_results_page import ApiTokenResultsPage
from .app_client_info import AppClientInfo
from .async_api_call import AsyncApiCall
from .async_api_call_output import AsyncApiCallOutput
from .async_api_call_results_page import AsyncApiCallResultsPage
from .async_api_call_type import AsyncApiCallType
from .axis import Axis
from .axis_direction_pair import AxisDirectionPair
from .billing_info import BillingInfo
from .cache_metadata import CacheMetadata
from .camera_drag_interaction_type import CameraDragInteractionType
from .card_details import CardDetails
from .cluster import Cluster
from .code_language import CodeLanguage
from .code_output import CodeOutput
from .commit import Commit
from .connection import Connection
from .country_code import CountryCode
from .coupon import Coupon
from .created_at_sort_mode import CreatedAtSortMode
from .currency import Currency
from .customer import Customer
from .customer_balance import CustomerBalance
from .device_access_token_request_form import DeviceAccessTokenRequestForm
from .device_auth_request_form import DeviceAuthRequestForm
from .device_auth_verify_params import DeviceAuthVerifyParams
from .direction import Direction
from .discount import Discount
from .docker_system_info import DockerSystemInfo
from .email_authentication_form import EmailAuthenticationForm
from .engine_metadata import EngineMetadata
from .environment import Environment
from .error import Error
from .executor_metadata import ExecutorMetadata
from .extended_user import ExtendedUser
from .extended_user_results_page import ExtendedUserResultsPage
from .extrude import Extrude
from .file_center_of_mass import FileCenterOfMass
from .file_conversion import FileConversion
from .file_density import FileDensity
from .file_export_format import FileExportFormat
from .file_import_format import FileImportFormat
from .file_mass import FileMass
from .file_surface_area import FileSurfaceArea
from .file_system_metadata import FileSystemMetadata
from .file_volume import FileVolume
from .gateway import Gateway
from .image_type import ImageType
from .index_info import IndexInfo
from .input_format import InputFormat
from .invoice import Invoice
from .invoice_line_item import InvoiceLineItem
from .invoice_status import InvoiceStatus
from .jetstream import Jetstream
from .jetstream_api_stats import JetstreamApiStats
from .jetstream_config import JetstreamConfig
from .jetstream_stats import JetstreamStats
from .leaf_node import LeafNode
from .mesh import Mesh
from .meta_cluster_info import MetaClusterInfo
from .metadata import Metadata
from .method import Method
from .modeling_cmd import ModelingCmd
from .modeling_cmd_id import ModelingCmdId
from .modeling_cmd_req import ModelingCmdReq
from .modeling_cmd_req_batch import ModelingCmdReqBatch
from .modeling_error import ModelingError
from .modeling_outcome import ModelingOutcome
from .modeling_outcomes import ModelingOutcomes
from .new_address import NewAddress
from .o_auth2_client_info import OAuth2ClientInfo
from .o_auth2_grant_type import OAuth2GrantType
from .onboarding import Onboarding
from .output_file import OutputFile
from .output_format import OutputFormat
from .path_segment import PathSegment
from .payment_intent import PaymentIntent
from .payment_method import PaymentMethod
from .payment_method_card_checks import PaymentMethodCardChecks
from .payment_method_type import PaymentMethodType
from .physics_constant import PhysicsConstant
from .physics_constant_name import PhysicsConstantName
from .plugins_info import PluginsInfo
from .point2d import Point2d
from .point3d import Point3d
from .point_e_metadata import PointEMetadata
from .pong import Pong
from .registry_service_config import RegistryServiceConfig
from .runtime import Runtime
from .session import Session
from .storage import Storage
from .system import System
from .system_info_cgroup_driver_enum import SystemInfoCgroupDriverEnum
from .system_info_cgroup_version_enum import SystemInfoCgroupVersionEnum
from .system_info_default_address_pools import SystemInfoDefaultAddressPools
from .system_info_isolation_enum import SystemInfoIsolationEnum
from .unit_angle import UnitAngle
from .unit_angle_conversion import UnitAngleConversion
from .unit_area import UnitArea
from .unit_area_conversion import UnitAreaConversion
from .unit_current import UnitCurrent
from .unit_current_conversion import UnitCurrentConversion
from .unit_energy import UnitEnergy
from .unit_energy_conversion import UnitEnergyConversion
from .unit_force import UnitForce
from .unit_force_conversion import UnitForceConversion
from .unit_frequency import UnitFrequency
from .unit_frequency_conversion import UnitFrequencyConversion
from .unit_length import UnitLength
from .unit_length_conversion import UnitLengthConversion
from .unit_mass import UnitMass
from .unit_mass_conversion import UnitMassConversion
from .unit_power import UnitPower
from .unit_power_conversion import UnitPowerConversion
from .unit_pressure import UnitPressure
from .unit_pressure_conversion import UnitPressureConversion
from .unit_temperature import UnitTemperature
from .unit_temperature_conversion import UnitTemperatureConversion
from .unit_torque import UnitTorque
from .unit_torque_conversion import UnitTorqueConversion
from .unit_volume import UnitVolume
from .unit_volume_conversion import UnitVolumeConversion
from .update_user import UpdateUser
from .user import User
from .user_results_page import UserResultsPage
from .uuid import Uuid
from .verification_token import VerificationToken



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Onboarding")


@attr.s(auto_attribs=True)
class Onboarding:
    """Onboarding details"""  # noqa: E501

    first_call_from__their_machine_date: Union[Unset, str] = UNSET
    first_litterbox_execute_date: Union[Unset, str] = UNSET
    first_token_date: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        first_call_from__their_machine_date = self.first_call_from__their_machine_date
        first_litterbox_execute_date = self.first_litterbox_execute_date
        first_token_date = self.first_token_date

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if first_call_from__their_machine_date is not UNSET:
            field_dict[
                "first_call_from_their_machine_date"
            ] = first_call_from__their_machine_date
        if first_litterbox_execute_date is not UNSET:
            field_dict["first_litterbox_execute_date"] = first_litterbox_execute_date
        if first_token_date is not UNSET:
            field_dict["first_token_date"] = first_token_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        first_call_from__their_machine_date = d.pop(
            "first_call_from_their_machine_date", UNSET
        )

        first_litterbox_execute_date = d.pop("first_litterbox_execute_date", UNSET)

        first_token_date = d.pop("first_token_date", UNSET)

        onboarding = cls(
            first_call_from__their_machine_date=first_call_from__their_machine_date,
            first_litterbox_execute_date=first_litterbox_execute_date,
            first_token_date=first_token_date,
        )

        onboarding.additional_properties = d
        return onboarding

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.api_call_status import ApiCallStatus
from ..models.file_export_format import FileExportFormat
from ..models.file_import_format import FileImportFormat
from ..models.input_format import InputFormat
from ..models.output_format import OutputFormat
from ..models.uuid import Uuid
from ..types import UNSET, Unset

F = TypeVar("F", bound="FileConversion")


@attr.s(auto_attribs=True)
class FileConversion:
    """A file conversion."""  # noqa: E501

    completed_at: Union[Unset, datetime.datetime] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    error: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    output: Union[Unset, str] = UNSET
    output_format: Union[Unset, FileExportFormat] = UNSET
    output_format_options: Union[Unset, OutputFormat] = UNSET
    outputs: Union[Unset, Any] = UNSET
    src_format: Union[Unset, FileImportFormat] = UNSET
    src_format_options: Union[Unset, InputFormat] = UNSET
    started_at: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, ApiCallStatus] = UNSET
    type: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        completed_at: Union[Unset, str] = UNSET
        if not isinstance(self.completed_at, Unset):
            completed_at = self.completed_at.isoformat()
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        error = self.error
        id = self.id
        output = self.output
        if not isinstance(self.output_format, Unset):
            output_format = self.output_format
        if not isinstance(self.output_format_options, Unset):
            output_format_options = self.output_format_options
        outputs = self.outputs
        if not isinstance(self.src_format, Unset):
            src_format = self.src_format
        if not isinstance(self.src_format_options, Unset):
            src_format_options = self.src_format_options
        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()
        if not isinstance(self.status, Unset):
            status = self.status
        type = self.type
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if error is not UNSET:
            field_dict["error"] = error
        if id is not UNSET:
            field_dict["id"] = id
        if output is not UNSET:
            field_dict["output"] = output
        if output_format is not UNSET:
            field_dict["output_format"] = output_format
        if output_format_options is not UNSET:
            field_dict["output_format_options"] = output_format_options
        if outputs is not UNSET:
            field_dict["outputs"] = outputs
        if src_format is not UNSET:
            field_dict["src_format"] = src_format
        if src_format_options is not UNSET:
            field_dict["src_format_options"] = src_format_options
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if status is not UNSET:
            field_dict["status"] = status
        if type is not UNSET:
            field_dict["type"] = type
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[F], src_dict: Dict[str, Any]) -> F:
        d = src_dict.copy()
        _completed_at = d.pop("completed_at", UNSET)
        completed_at: Union[Unset, datetime.datetime]
        if isinstance(_completed_at, Unset):
            completed_at = UNSET
        else:
            completed_at = isoparse(_completed_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        error = d.pop("error", UNSET)

        _id = d.pop("id", UNSET)
        id: Union[Unset, Uuid]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = Uuid(_id)

        output = d.pop("output", UNSET)

        _output_format = d.pop("output_format", UNSET)
        output_format: Union[Unset, FileExportFormat]
        if isinstance(_output_format, Unset):
            output_format = UNSET
        else:
            output_format = _output_format  # type: ignore[arg-type]

        _output_format_options = d.pop("output_format_options", UNSET)
        output_format_options: Union[Unset, OutputFormat]
        if isinstance(_output_format_options, Unset):
            output_format_options = UNSET
        else:
            output_format_options = OutputFormat(_output_format_options)

        outputs = d.pop("outputs", UNSET)
        _src_format = d.pop("src_format", UNSET)
        src_format: Union[Unset, FileImportFormat]
        if isinstance(_src_format, Unset):
            src_format = UNSET
        else:
            src_format = _src_format  # type: ignore[arg-type]

        _src_format_options = d.pop("src_format_options", UNSET)
        src_format_options: Union[Unset, InputFormat]
        if isinstance(_src_format_options, Unset):
            src_format_options = UNSET
        else:
            src_format_options = InputFormat(_src_format_options)

        _started_at = d.pop("started_at", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ApiCallStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = _status  # type: ignore[arg-type]

        type = d.pop("type", UNSET)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_id = d.pop("user_id", UNSET)

        file_conversion = cls(
            completed_at=completed_at,
            created_at=created_at,
            error=error,
            id=id,
            output=output,
            output_format=output_format,
            output_format_options=output_format_options,
            outputs=outputs,
            src_format=src_format,
            src_format_options=src_format_options,
            started_at=started_at,
            status=status,
            type=type,
            updated_at=updated_at,
            user_id=user_id,
        )

        file_conversion.additional_properties = d
        return file_conversion

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


B = TypeVar("B", bound="FileCenterOfMass")


@attr.s(auto_attribs=True)
class FileCenterOfMass:
    """File center of mass."""  # noqa: E501

    center_of_mass: Union[Unset, List[float]] = UNSET
    completed_at: Union[Unset, datetime.datetime] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    error: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    src_format: Union[Unset, FileImportFormat] = UNSET
    started_at: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, ApiCallStatus] = UNSET
    type: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        center_of_mass: Union[Unset, List[float]] = UNSET
        if not isinstance(self.center_of_mass, Unset):
            center_of_mass = self.center_of_mass
        completed_at: Union[Unset, str] = UNSET
        if not isinstance(self.completed_at, Unset):
            completed_at = self.completed_at.isoformat()
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        error = self.error
        id = self.id
        if not isinstance(self.src_format, Unset):
            src_format = self.src_format
        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()
        if not isinstance(self.status, Unset):
            status = self.status
        type = self.type
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if center_of_mass is not UNSET:
            field_dict["center_of_mass"] = center_of_mass
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if error is not UNSET:
            field_dict["error"] = error
        if id is not UNSET:
            field_dict["id"] = id
        if src_format is not UNSET:
            field_dict["src_format"] = src_format
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if status is not UNSET:
            field_dict["status"] = status
        if type is not UNSET:
            field_dict["type"] = type
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[B], src_dict: Dict[str, Any]) -> B:
        d = src_dict.copy()
        center_of_mass = cast(List[float], d.pop("center_of_mass", UNSET))

        _completed_at = d.pop("completed_at", UNSET)
        completed_at: Union[Unset, datetime.datetime]
        if isinstance(_completed_at, Unset):
            completed_at = UNSET
        else:
            completed_at = isoparse(_completed_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        error = d.pop("error", UNSET)

        _id = d.pop("id", UNSET)
        id: Union[Unset, Uuid]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = Uuid(_id)

        _src_format = d.pop("src_format", UNSET)
        src_format: Union[Unset, FileImportFormat]
        if isinstance(_src_format, Unset):
            src_format = UNSET
        else:
            src_format = _src_format  # type: ignore[arg-type]

        _started_at = d.pop("started_at", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ApiCallStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = _status  # type: ignore[arg-type]

        type = d.pop("type", UNSET)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_id = d.pop("user_id", UNSET)

        file_center_of_mass = cls(
            center_of_mass=center_of_mass,
            completed_at=completed_at,
            created_at=created_at,
            error=error,
            id=id,
            src_format=src_format,
            started_at=started_at,
            status=status,
            type=type,
            updated_at=updated_at,
            user_id=user_id,
        )

        file_center_of_mass.additional_properties = d
        return file_center_of_mass

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


Q = TypeVar("Q", bound="FileMass")


@attr.s(auto_attribs=True)
class FileMass:
    """A file mass."""  # noqa: E501

    completed_at: Union[Unset, datetime.datetime] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    error: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    mass: Union[Unset, float] = UNSET
    material_density: Union[Unset, float] = UNSET
    src_format: Union[Unset, FileImportFormat] = UNSET
    started_at: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, ApiCallStatus] = UNSET
    type: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        completed_at: Union[Unset, str] = UNSET
        if not isinstance(self.completed_at, Unset):
            completed_at = self.completed_at.isoformat()
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        error = self.error
        id = self.id
        mass = self.mass
        material_density = self.material_density
        if not isinstance(self.src_format, Unset):
            src_format = self.src_format
        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()
        if not isinstance(self.status, Unset):
            status = self.status
        type = self.type
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if error is not UNSET:
            field_dict["error"] = error
        if id is not UNSET:
            field_dict["id"] = id
        if mass is not UNSET:
            field_dict["mass"] = mass
        if material_density is not UNSET:
            field_dict["material_density"] = material_density
        if src_format is not UNSET:
            field_dict["src_format"] = src_format
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if status is not UNSET:
            field_dict["status"] = status
        if type is not UNSET:
            field_dict["type"] = type
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[Q], src_dict: Dict[str, Any]) -> Q:
        d = src_dict.copy()
        _completed_at = d.pop("completed_at", UNSET)
        completed_at: Union[Unset, datetime.datetime]
        if isinstance(_completed_at, Unset):
            completed_at = UNSET
        else:
            completed_at = isoparse(_completed_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        error = d.pop("error", UNSET)

        _id = d.pop("id", UNSET)
        id: Union[Unset, Uuid]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = Uuid(_id)

        mass = d.pop("mass", UNSET)

        material_density = d.pop("material_density", UNSET)

        _src_format = d.pop("src_format", UNSET)
        src_format: Union[Unset, FileImportFormat]
        if isinstance(_src_format, Unset):
            src_format = UNSET
        else:
            src_format = _src_format  # type: ignore[arg-type]

        _started_at = d.pop("started_at", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ApiCallStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = _status  # type: ignore[arg-type]

        type = d.pop("type", UNSET)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_id = d.pop("user_id", UNSET)

        file_mass = cls(
            completed_at=completed_at,
            created_at=created_at,
            error=error,
            id=id,
            mass=mass,
            material_density=material_density,
            src_format=src_format,
            started_at=started_at,
            status=status,
            type=type,
            updated_at=updated_at,
            user_id=user_id,
        )

        file_mass.additional_properties = d
        return file_mass

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


P = TypeVar("P", bound="FileVolume")


@attr.s(auto_attribs=True)
class FileVolume:
    """A file volume."""  # noqa: E501

    completed_at: Union[Unset, datetime.datetime] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    error: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    src_format: Union[Unset, FileImportFormat] = UNSET
    started_at: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, ApiCallStatus] = UNSET
    type: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_id: Union[Unset, str] = UNSET
    volume: Union[Unset, float] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        completed_at: Union[Unset, str] = UNSET
        if not isinstance(self.completed_at, Unset):
            completed_at = self.completed_at.isoformat()
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        error = self.error
        id = self.id
        if not isinstance(self.src_format, Unset):
            src_format = self.src_format
        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()
        if not isinstance(self.status, Unset):
            status = self.status
        type = self.type
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_id = self.user_id
        volume = self.volume

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if error is not UNSET:
            field_dict["error"] = error
        if id is not UNSET:
            field_dict["id"] = id
        if src_format is not UNSET:
            field_dict["src_format"] = src_format
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if status is not UNSET:
            field_dict["status"] = status
        if type is not UNSET:
            field_dict["type"] = type
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if user_id is not UNSET:
            field_dict["user_id"] = user_id
        if volume is not UNSET:
            field_dict["volume"] = volume

        return field_dict

    @classmethod
    def from_dict(cls: Type[P], src_dict: Dict[str, Any]) -> P:
        d = src_dict.copy()
        _completed_at = d.pop("completed_at", UNSET)
        completed_at: Union[Unset, datetime.datetime]
        if isinstance(_completed_at, Unset):
            completed_at = UNSET
        else:
            completed_at = isoparse(_completed_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        error = d.pop("error", UNSET)

        _id = d.pop("id", UNSET)
        id: Union[Unset, Uuid]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = Uuid(_id)

        _src_format = d.pop("src_format", UNSET)
        src_format: Union[Unset, FileImportFormat]
        if isinstance(_src_format, Unset):
            src_format = UNSET
        else:
            src_format = _src_format  # type: ignore[arg-type]

        _started_at = d.pop("started_at", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ApiCallStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = _status  # type: ignore[arg-type]

        type = d.pop("type", UNSET)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_id = d.pop("user_id", UNSET)

        volume = d.pop("volume", UNSET)

        file_volume = cls(
            completed_at=completed_at,
            created_at=created_at,
            error=error,
            id=id,
            src_format=src_format,
            started_at=started_at,
            status=status,
            type=type,
            updated_at=updated_at,
            user_id=user_id,
            volume=volume,
        )

        file_volume.additional_properties = d
        return file_volume

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


K = TypeVar("K", bound="FileDensity")


@attr.s(auto_attribs=True)
class FileDensity:
    """A file density."""  # noqa: E501

    completed_at: Union[Unset, datetime.datetime] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    density: Union[Unset, float] = UNSET
    error: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    material_mass: Union[Unset, float] = UNSET
    src_format: Union[Unset, FileImportFormat] = UNSET
    started_at: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, ApiCallStatus] = UNSET
    type: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        completed_at: Union[Unset, str] = UNSET
        if not isinstance(self.completed_at, Unset):
            completed_at = self.completed_at.isoformat()
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        density = self.density
        error = self.error
        id = self.id
        material_mass = self.material_mass
        if not isinstance(self.src_format, Unset):
            src_format = self.src_format
        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()
        if not isinstance(self.status, Unset):
            status = self.status
        type = self.type
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if density is not UNSET:
            field_dict["density"] = density
        if error is not UNSET:
            field_dict["error"] = error
        if id is not UNSET:
            field_dict["id"] = id
        if material_mass is not UNSET:
            field_dict["material_mass"] = material_mass
        if src_format is not UNSET:
            field_dict["src_format"] = src_format
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if status is not UNSET:
            field_dict["status"] = status
        if type is not UNSET:
            field_dict["type"] = type
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[K], src_dict: Dict[str, Any]) -> K:
        d = src_dict.copy()
        _completed_at = d.pop("completed_at", UNSET)
        completed_at: Union[Unset, datetime.datetime]
        if isinstance(_completed_at, Unset):
            completed_at = UNSET
        else:
            completed_at = isoparse(_completed_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        density = d.pop("density", UNSET)

        error = d.pop("error", UNSET)

        _id = d.pop("id", UNSET)
        id: Union[Unset, Uuid]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = Uuid(_id)

        material_mass = d.pop("material_mass", UNSET)

        _src_format = d.pop("src_format", UNSET)
        src_format: Union[Unset, FileImportFormat]
        if isinstance(_src_format, Unset):
            src_format = UNSET
        else:
            src_format = _src_format  # type: ignore[arg-type]

        _started_at = d.pop("started_at", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ApiCallStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = _status  # type: ignore[arg-type]

        type = d.pop("type", UNSET)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_id = d.pop("user_id", UNSET)

        file_density = cls(
            completed_at=completed_at,
            created_at=created_at,
            density=density,
            error=error,
            id=id,
            material_mass=material_mass,
            src_format=src_format,
            started_at=started_at,
            status=status,
            type=type,
            updated_at=updated_at,
            user_id=user_id,
        )

        file_density.additional_properties = d
        return file_density

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


C = TypeVar("C", bound="FileSurfaceArea")


@attr.s(auto_attribs=True)
class FileSurfaceArea:
    """A file surface area."""  # noqa: E501

    completed_at: Union[Unset, datetime.datetime] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    error: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    src_format: Union[Unset, FileImportFormat] = UNSET
    started_at: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, ApiCallStatus] = UNSET
    surface_area: Union[Unset, float] = UNSET
    type: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        completed_at: Union[Unset, str] = UNSET
        if not isinstance(self.completed_at, Unset):
            completed_at = self.completed_at.isoformat()
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        error = self.error
        id = self.id
        if not isinstance(self.src_format, Unset):
            src_format = self.src_format
        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()
        if not isinstance(self.status, Unset):
            status = self.status
        surface_area = self.surface_area
        type = self.type
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if error is not UNSET:
            field_dict["error"] = error
        if id is not UNSET:
            field_dict["id"] = id
        if src_format is not UNSET:
            field_dict["src_format"] = src_format
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if status is not UNSET:
            field_dict["status"] = status
        if surface_area is not UNSET:
            field_dict["surface_area"] = surface_area
        if type is not UNSET:
            field_dict["type"] = type
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[C], src_dict: Dict[str, Any]) -> C:
        d = src_dict.copy()
        _completed_at = d.pop("completed_at", UNSET)
        completed_at: Union[Unset, datetime.datetime]
        if isinstance(_completed_at, Unset):
            completed_at = UNSET
        else:
            completed_at = isoparse(_completed_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        error = d.pop("error", UNSET)

        _id = d.pop("id", UNSET)
        id: Union[Unset, Uuid]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = Uuid(_id)

        _src_format = d.pop("src_format", UNSET)
        src_format: Union[Unset, FileImportFormat]
        if isinstance(_src_format, Unset):
            src_format = UNSET
        else:
            src_format = _src_format  # type: ignore[arg-type]

        _started_at = d.pop("started_at", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ApiCallStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = _status  # type: ignore[arg-type]

        surface_area = d.pop("surface_area", UNSET)

        type = d.pop("type", UNSET)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_id = d.pop("user_id", UNSET)

        file_surface_area = cls(
            completed_at=completed_at,
            created_at=created_at,
            error=error,
            id=id,
            src_format=src_format,
            started_at=started_at,
            status=status,
            surface_area=surface_area,
            type=type,
            updated_at=updated_at,
            user_id=user_id,
        )

        file_surface_area.additional_properties = d
        return file_surface_area

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


AsyncApiCallOutput = Union[
    FileConversion, FileCenterOfMass, FileMass, FileVolume, FileDensity, FileSurfaceArea
]



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.api_call_status import ApiCallStatus
from ..models.unit_force import UnitForce
from ..models.uuid import Uuid
from ..types import UNSET, Unset

Y = TypeVar("Y", bound="UnitForceConversion")


@attr.s(auto_attribs=True)
class UnitForceConversion:
    """Result of converting between units."""  # noqa: E501

    completed_at: Union[Unset, datetime.datetime] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    error: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    input: Union[Unset, float] = UNSET
    input_unit: Union[Unset, UnitForce] = UNSET
    output: Union[Unset, float] = UNSET
    output_unit: Union[Unset, UnitForce] = UNSET
    started_at: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, ApiCallStatus] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        completed_at: Union[Unset, str] = UNSET
        if not isinstance(self.completed_at, Unset):
            completed_at = self.completed_at.isoformat()
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        error = self.error
        id = self.id
        input = self.input
        if not isinstance(self.input_unit, Unset):
            input_unit = self.input_unit
        output = self.output
        if not isinstance(self.output_unit, Unset):
            output_unit = self.output_unit
        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()
        if not isinstance(self.status, Unset):
            status = self.status
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if error is not UNSET:
            field_dict["error"] = error
        if id is not UNSET:
            field_dict["id"] = id
        if input is not UNSET:
            field_dict["input"] = input
        if input_unit is not UNSET:
            field_dict["input_unit"] = input_unit
        if output is not UNSET:
            field_dict["output"] = output
        if output_unit is not UNSET:
            field_dict["output_unit"] = output_unit
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if status is not UNSET:
            field_dict["status"] = status
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[Y], src_dict: Dict[str, Any]) -> Y:
        d = src_dict.copy()
        _completed_at = d.pop("completed_at", UNSET)
        completed_at: Union[Unset, datetime.datetime]
        if isinstance(_completed_at, Unset):
            completed_at = UNSET
        else:
            completed_at = isoparse(_completed_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        error = d.pop("error", UNSET)

        _id = d.pop("id", UNSET)
        id: Union[Unset, Uuid]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = Uuid(_id)

        input = d.pop("input", UNSET)

        _input_unit = d.pop("input_unit", UNSET)
        input_unit: Union[Unset, UnitForce]
        if isinstance(_input_unit, Unset):
            input_unit = UNSET
        else:
            input_unit = _input_unit  # type: ignore[arg-type]

        output = d.pop("output", UNSET)

        _output_unit = d.pop("output_unit", UNSET)
        output_unit: Union[Unset, UnitForce]
        if isinstance(_output_unit, Unset):
            output_unit = UNSET
        else:
            output_unit = _output_unit  # type: ignore[arg-type]

        _started_at = d.pop("started_at", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ApiCallStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = _status  # type: ignore[arg-type]

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_id = d.pop("user_id", UNSET)

        unit_force_conversion = cls(
            completed_at=completed_at,
            created_at=created_at,
            error=error,
            id=id,
            input=input,
            input_unit=input_unit,
            output=output,
            output_unit=output_unit,
            started_at=started_at,
            status=status,
            updated_at=updated_at,
            user_id=user_id,
        )

        unit_force_conversion.additional_properties = d
        return unit_force_conversion

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

N = TypeVar("N", bound="PaymentMethodCardChecks")


@attr.s(auto_attribs=True)
class PaymentMethodCardChecks:
    """Card checks."""  # noqa: E501

    address_line1_check: Union[Unset, str] = UNSET
    address_postal_code_check: Union[Unset, str] = UNSET
    cvc_check: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        address_line1_check = self.address_line1_check
        address_postal_code_check = self.address_postal_code_check
        cvc_check = self.cvc_check

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if address_line1_check is not UNSET:
            field_dict["address_line1_check"] = address_line1_check
        if address_postal_code_check is not UNSET:
            field_dict["address_postal_code_check"] = address_postal_code_check
        if cvc_check is not UNSET:
            field_dict["cvc_check"] = cvc_check

        return field_dict

    @classmethod
    def from_dict(cls: Type[N], src_dict: Dict[str, Any]) -> N:
        d = src_dict.copy()
        address_line1_check = d.pop("address_line1_check", UNSET)

        address_postal_code_check = d.pop("address_postal_code_check", UNSET)

        cvc_check = d.pop("cvc_check", UNSET)

        payment_method_card_checks = cls(
            address_line1_check=address_line1_check,
            address_postal_code_check=address_postal_code_check,
            cvc_check=cvc_check,
        )

        payment_method_card_checks.additional_properties = d
        return payment_method_card_checks

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.docker_system_info import DockerSystemInfo
from ..models.environment import Environment
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExecutorMetadata")


@attr.s(auto_attribs=True)
class ExecutorMetadata:
    """Metadata about our currently running server.

    This is mostly used for internal purposes and debugging."""  # noqa: E501

    docker_info: Union[Unset, DockerSystemInfo] = UNSET
    environment: Union[Unset, Environment] = UNSET
    git_hash: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.docker_info, Unset):
            docker_info = self.docker_info
        if not isinstance(self.environment, Unset):
            environment = self.environment
        git_hash = self.git_hash

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if docker_info is not UNSET:
            field_dict["docker_info"] = docker_info
        if environment is not UNSET:
            field_dict["environment"] = environment
        if git_hash is not UNSET:
            field_dict["git_hash"] = git_hash

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _docker_info = d.pop("docker_info", UNSET)
        docker_info: Union[Unset, DockerSystemInfo]
        if isinstance(_docker_info, Unset):
            docker_info = UNSET
        else:
            docker_info = DockerSystemInfo(_docker_info)

        _environment = d.pop("environment", UNSET)
        environment: Union[Unset, Environment]
        if isinstance(_environment, Unset):
            environment = UNSET
        else:
            environment = _environment  # type: ignore[arg-type]

        git_hash = d.pop("git_hash", UNSET)

        executor_metadata = cls(
            docker_info=docker_info,
            environment=environment,
            git_hash=git_hash,
        )

        executor_metadata.additional_properties = d
        return executor_metadata

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.uuid import Uuid
from ..types import UNSET, Unset

F = TypeVar("F", bound="Session")


@attr.s(auto_attribs=True)
class Session:
    """An authentication session.

    For our UIs, these are automatically created by Next.js."""  # noqa: E501

    created_at: Union[Unset, datetime.datetime] = UNSET
    expires: Union[Unset, datetime.datetime] = UNSET
    id: Union[Unset, str] = UNSET
    session_token: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        expires: Union[Unset, str] = UNSET
        if not isinstance(self.expires, Unset):
            expires = self.expires.isoformat()
        id = self.id
        session_token = self.session_token
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if expires is not UNSET:
            field_dict["expires"] = expires
        if id is not UNSET:
            field_dict["id"] = id
        if session_token is not UNSET:
            field_dict["session_token"] = session_token
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[F], src_dict: Dict[str, Any]) -> F:
        d = src_dict.copy()
        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _expires = d.pop("expires", UNSET)
        expires: Union[Unset, datetime.datetime]
        if isinstance(_expires, Unset):
            expires = UNSET
        else:
            expires = isoparse(_expires)

        id = d.pop("id", UNSET)

        _session_token = d.pop("session_token", UNSET)
        session_token: Union[Unset, Uuid]
        if isinstance(_session_token, Unset):
            session_token = UNSET
        else:
            session_token = Uuid(_session_token)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_id = d.pop("user_id", UNSET)

        session = cls(
            created_at=created_at,
            expires=expires,
            id=id,
            session_token=session_token,
            updated_at=updated_at,
            user_id=user_id,
        )

        session.additional_properties = d
        return session

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.error import Error
from ...types import Response


def _get_kwargs(
    email: str,
    token: str,
    *,
    client: Client,
    callback_url: Optional[str] = None,
) -> Dict[str, Any]:
    url = "{}/auth/email/callback".format(client.base_url)  # noqa: E501
    if callback_url is not None:
        if "?" in url:
            url = url + "&callback_url=" + str(callback_url)
        else:
            url = url + "?callback_url=" + str(callback_url)
    if email is not None:
        if "?" in url:
            url = url + "&email=" + str(email)
        else:
            url = url + "?email=" + str(email)
    if token is not None:
        if "?" in url:
            url = url + "&token=" + str(token)
        else:
            url = url + "?token=" + str(token)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Error]:
    return None
    if response.status_code == 400:
        response_4XX = Error.from_dict(response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error.from_dict(response.json())
        return response_5XX
    return Error.from_dict(response.json())


def _build_response(*, response: httpx.Response) -> Response[Optional[Error]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    email: str,
    token: str,
    *,
    client: Client,
    callback_url: Optional[str] = None,
) -> Response[Optional[Error]]:
    kwargs = _get_kwargs(
        callback_url=callback_url,
        email=email,
        token=token,
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    email: str,
    token: str,
    *,
    client: Client,
    callback_url: Optional[str] = None,
) -> Optional[Error]:

    return sync_detailed(
        callback_url=callback_url,
        email=email,
        token=token,
        client=client,
    ).parsed


async def asyncio_detailed(
    email: str,
    token: str,
    *,
    client: Client,
    callback_url: Optional[str] = None,
) -> Response[Optional[Error]]:
    kwargs = _get_kwargs(
        callback_url=callback_url,
        email=email,
        token=token,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    email: str,
    token: str,
    *,
    client: Client,
    callback_url: Optional[str] = None,
) -> Optional[Error]:

    return (
        await asyncio_detailed(
            callback_url=callback_url,
            email=email,
            token=token,
            client=client,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

H = TypeVar("H", bound="AsyncApiCallResultsPage")


@attr.s(auto_attribs=True)
class AsyncApiCallResultsPage:
    """A single page of results"""  # noqa: E501

    from ..models.async_api_call import AsyncApiCall

    items: Union[Unset, List[AsyncApiCall]] = UNSET
    next_page: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.async_api_call import AsyncApiCall

        items: Union[Unset, List[AsyncApiCall]] = UNSET
        if not isinstance(self.items, Unset):
            items = self.items
        next_page = self.next_page

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if items is not UNSET:
            field_dict["items"] = items
        if next_page is not UNSET:
            field_dict["next_page"] = next_page

        return field_dict

    @classmethod
    def from_dict(cls: Type[H], src_dict: Dict[str, Any]) -> H:
        d = src_dict.copy()
        from ..models.async_api_call import AsyncApiCall

        items = cast(List[AsyncApiCall], d.pop("items", UNSET))

        next_page = d.pop("next_page", UNSET)

        async_api_call_results_page = cls(
            items=items,
            next_page=next_page,
        )

        async_api_call_results_page.additional_properties = d
        return async_api_call_results_page

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

J = TypeVar("J", bound="Runtime")


@attr.s(auto_attribs=True)
class Runtime:
    """Runtime describes an [OCI compliant](https://github.com/opencontainers/runtime-spec) runtime.  The runtime is invoked by the daemon via the `containerd` daemon. OCI runtimes act as an interface to the Linux kernel namespaces, cgroups, and SELinux."""  # noqa: E501

    path: Union[Unset, str] = UNSET
    runtime_args: Union[Unset, List[str]] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        path = self.path
        runtime_args: Union[Unset, List[str]] = UNSET
        if not isinstance(self.runtime_args, Unset):
            runtime_args = self.runtime_args

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if path is not UNSET:
            field_dict["path"] = path
        if runtime_args is not UNSET:
            field_dict["runtime_args"] = runtime_args

        return field_dict

    @classmethod
    def from_dict(cls: Type[J], src_dict: Dict[str, Any]) -> J:
        d = src_dict.copy()
        path = d.pop("path", UNSET)

        runtime_args = cast(List[str], d.pop("runtime_args", UNSET))

        runtime = cls(
            path=path,
            runtime_args=runtime_args,
        )

        runtime.additional_properties = d
        return runtime

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.api_call_status import ApiCallStatus
from ..models.file_import_format import FileImportFormat
from ..models.uuid import Uuid
from ..types import UNSET, Unset

A = TypeVar("A", bound="FileCenterOfMass")


@attr.s(auto_attribs=True)
class FileCenterOfMass:
    """A file center of mass result."""  # noqa: E501

    center_of_mass: Union[Unset, List[float]] = UNSET
    completed_at: Union[Unset, datetime.datetime] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    error: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    src_format: Union[Unset, FileImportFormat] = UNSET
    started_at: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, ApiCallStatus] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        center_of_mass: Union[Unset, List[float]] = UNSET
        if not isinstance(self.center_of_mass, Unset):
            center_of_mass = self.center_of_mass
        completed_at: Union[Unset, str] = UNSET
        if not isinstance(self.completed_at, Unset):
            completed_at = self.completed_at.isoformat()
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        error = self.error
        id = self.id
        if not isinstance(self.src_format, Unset):
            src_format = self.src_format
        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()
        if not isinstance(self.status, Unset):
            status = self.status
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if center_of_mass is not UNSET:
            field_dict["center_of_mass"] = center_of_mass
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if error is not UNSET:
            field_dict["error"] = error
        if id is not UNSET:
            field_dict["id"] = id
        if src_format is not UNSET:
            field_dict["src_format"] = src_format
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if status is not UNSET:
            field_dict["status"] = status
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[A], src_dict: Dict[str, Any]) -> A:
        d = src_dict.copy()
        center_of_mass = cast(List[float], d.pop("center_of_mass", UNSET))

        _completed_at = d.pop("completed_at", UNSET)
        completed_at: Union[Unset, datetime.datetime]
        if isinstance(_completed_at, Unset):
            completed_at = UNSET
        else:
            completed_at = isoparse(_completed_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        error = d.pop("error", UNSET)

        _id = d.pop("id", UNSET)
        id: Union[Unset, Uuid]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = Uuid(_id)

        _src_format = d.pop("src_format", UNSET)
        src_format: Union[Unset, FileImportFormat]
        if isinstance(_src_format, Unset):
            src_format = UNSET
        else:
            src_format = _src_format  # type: ignore[arg-type]

        _started_at = d.pop("started_at", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ApiCallStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = _status  # type: ignore[arg-type]

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_id = d.pop("user_id", UNSET)

        file_center_of_mass = cls(
            center_of_mass=center_of_mass,
            completed_at=completed_at,
            created_at=created_at,
            error=error,
            id=id,
            src_format=src_format,
            started_at=started_at,
            status=status,
            updated_at=updated_at,
            user_id=user_id,
        )

        file_center_of_mass.additional_properties = d
        return file_center_of_mass

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.error import Error
from ...models.file_center_of_mass import FileCenterOfMass
from ...models.file_conversion import FileConversion
from ...models.file_density import FileDensity
from ...models.file_mass import FileMass
from ...models.file_surface_area import FileSurfaceArea
from ...models.file_volume import FileVolume
from ...types import Response


def _get_kwargs(
    id: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/async/operations/{id}".format(client.base_url, id=id)  # noqa: E501

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        FileConversion,
        FileCenterOfMass,
        FileMass,
        FileVolume,
        FileDensity,
        FileSurfaceArea,
        Error,
    ]
]:
    if response.status_code == 200:
        data = response.json()
        try:
            if not isinstance(data, dict):
                raise TypeError()
            option_file_conversion = FileConversion.from_dict(data)
            return option_file_conversion
        except ValueError:
            pass
        except TypeError:
            pass
        try:
            if not isinstance(data, dict):
                raise TypeError()
            option_file_center_of_mass = FileCenterOfMass.from_dict(data)
            return option_file_center_of_mass
        except ValueError:
            pass
        except TypeError:
            pass
        try:
            if not isinstance(data, dict):
                raise TypeError()
            option_file_mass = FileMass.from_dict(data)
            return option_file_mass
        except ValueError:
            pass
        except TypeError:
            pass
        try:
            if not isinstance(data, dict):
                raise TypeError()
            option_file_volume = FileVolume.from_dict(data)
            return option_file_volume
        except ValueError:
            pass
        except TypeError:
            pass
        try:
            if not isinstance(data, dict):
                raise TypeError()
            option_file_density = FileDensity.from_dict(data)
            return option_file_density
        except ValueError:
            pass
        except TypeError:
            pass
        try:
            if not isinstance(data, dict):
                raise TypeError()
            option_file_surface_area = FileSurfaceArea.from_dict(data)
            return option_file_surface_area
        except ValueError:
            raise
        except TypeError:
            raise
    if response.status_code == 400:
        response_4XX = Error.from_dict(response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error.from_dict(response.json())
        return response_5XX
    return Error.from_dict(response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[
    Optional[
        Union[
            FileConversion,
            FileCenterOfMass,
            FileMass,
            FileVolume,
            FileDensity,
            FileSurfaceArea,
            Error,
        ]
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: Client,
) -> Response[
    Optional[
        Union[
            FileConversion,
            FileCenterOfMass,
            FileMass,
            FileVolume,
            FileDensity,
            FileSurfaceArea,
            Error,
        ]
    ]
]:
    kwargs = _get_kwargs(
        id=id,
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    id: str,
    *,
    client: Client,
) -> Optional[
    Union[
        FileConversion,
        FileCenterOfMass,
        FileMass,
        FileVolume,
        FileDensity,
        FileSurfaceArea,
        Error,
    ]
]:
    """Get the status and output of an async operation.
    This endpoint requires authentication by any KittyCAD user. It returns details of the requested async operation for the user.
    If the user is not authenticated to view the specified async operation, then it is not returned.
    Only KittyCAD employees with the proper access can view async operations for other users."""  # noqa: E501

    return sync_detailed(
        id=id,
        client=client,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: Client,
) -> Response[
    Optional[
        Union[
            FileConversion,
            FileCenterOfMass,
            FileMass,
            FileVolume,
            FileDensity,
            FileSurfaceArea,
            Error,
        ]
    ]
]:
    kwargs = _get_kwargs(
        id=id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    id: str,
    *,
    client: Client,
) -> Optional[
    Union[
        FileConversion,
        FileCenterOfMass,
        FileMass,
        FileVolume,
        FileDensity,
        FileSurfaceArea,
        Error,
    ]
]:
    """Get the status and output of an async operation.
    This endpoint requires authentication by any KittyCAD user. It returns details of the requested async operation for the user.
    If the user is not authenticated to view the specified async operation, then it is not returned.
    Only KittyCAD employees with the proper access can view async operations for other users."""  # noqa: E501

    return (
        await asyncio_detailed(
            id=id,
            client=client,
        )
    ).parsed



import os
from typing import Union

import pytest

from .api.api_tokens import list_api_tokens_for_user
from .api.file import (
    create_file_conversion_with_base64_helper,
    create_file_mass,
    create_file_volume,
)
from .api.meta import ping
from .api.users import get_user_self, list_users_extended
from .client import ClientFromEnv
from .models import (
    ApiCallStatus,
    ApiTokenResultsPage,
    CreatedAtSortMode,
    Error,
    ExtendedUserResultsPage,
    FileConversion,
    FileExportFormat,
    FileImportFormat,
    FileMass,
    FileVolume,
    Pong,
    User,
)


def test_get_session():
    # Create our client.
    client = ClientFromEnv()

    # Get the session.
    session: Union[User, Error, None] = get_user_self.sync(client=client)

    assert isinstance(session, User)

    print(f"Session: {session}")


@pytest.mark.asyncio
async def test_get_api_tokens_async():
    # Create our client.
    client = ClientFromEnv()

    # List API tokens.
    fc: Union[ApiTokenResultsPage, Error, None] = list_api_tokens_for_user.sync(
        client=client, sort_by=CreatedAtSortMode.CREATED_AT_ASCENDING
    )

    assert isinstance(fc, ApiTokenResultsPage)

    print(f"fc: {fc}")


@pytest.mark.asyncio
async def test_get_session_async():
    # Create our client.
    client = ClientFromEnv()

    # Get the session.
    session: Union[User, Error, None] = await get_user_self.asyncio(client=client)

    assert isinstance(session, User)

    print(f"Session: {session}")


def test_ping():
    # Create our client.
    client = ClientFromEnv()

    # Get the message.
    message: Union[Pong, Error, None] = ping.sync(client=client)

    assert isinstance(message, Pong)

    print(f"Message: {message}")


@pytest.mark.asyncio
async def test_ping_async():
    # Create our client.
    client = ClientFromEnv()

    # Get the message.
    message: Union[Pong, Error, None] = await ping.asyncio(client=client)

    assert isinstance(message, Pong)

    print(f"Message: {message}")


def test_file_convert_stl():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.stl"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    result: Union[
        FileConversion, Error, None
    ] = create_file_conversion_with_base64_helper.sync(
        client=client,
        body=content,
        src_format=FileImportFormat.STL,
        output_format=FileExportFormat.OBJ,
    )

    assert isinstance(result, FileConversion)

    fc: FileConversion = result

    print(f"FileConversion: {fc}")

    assert fc.id is not None

    assert fc.status == ApiCallStatus.COMPLETED

    print(f"FileConversion: {fc}")


@pytest.mark.asyncio
async def test_file_convert_stl_async():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.stl"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    result: Union[
        FileConversion, Error, None
    ] = await create_file_conversion_with_base64_helper.asyncio(
        client=client,
        body=content,
        src_format=FileImportFormat.STL,
        output_format=FileExportFormat.OBJ,
    )

    assert isinstance(result, FileConversion)

    fc: FileConversion = result

    print(f"FileConversion: {fc}")

    assert fc.id is not None

    print(f"FileConversion: {fc}")


def test_file_mass():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.obj"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    result: Union[FileMass, Error, None] = create_file_mass.sync(
        client=client,
        body=content,
        src_format=FileImportFormat.OBJ,
        material_density=1.0,
    )

    assert isinstance(result, FileMass)

    fm: FileMass = result

    print(f"FileMass: {fm}")

    assert fm.id is not None
    assert fm.mass is not None

    assert fm.to_dict() is not None

    assert fm.status == ApiCallStatus.COMPLETED


def test_file_volume():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.obj"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    result: Union[FileVolume, Error, None] = create_file_volume.sync(
        client=client, body=content, src_format=FileImportFormat.OBJ
    )

    assert isinstance(result, FileVolume)

    fv: FileVolume = result

    print(f"FileVolume: {fv}")

    assert fv.id is not None
    assert fv.volume is not None

    assert fv.to_dict() is not None

    assert fv.status == ApiCallStatus.COMPLETED


def test_list_users():
    # Create our client.
    client = ClientFromEnv()

    response: Union[ExtendedUserResultsPage, Error, None] = list_users_extended.sync(
        sort_by=CreatedAtSortMode.CREATED_AT_DESCENDING, client=client, limit=10
    )

    assert isinstance(response, ExtendedUserResultsPage)

    print(f"ExtendedUserResultsPage: {response}")



from enum import Enum


class UnitArea(str, Enum):
    """The valid types of area units."""  # noqa: E501

    """# Acres <https://en.wikipedia.org/wiki/Acre> """  # noqa: E501
    ACRES = "acres"
    """# Hectares <https://en.wikipedia.org/wiki/Hectare> """  # noqa: E501
    HECTARES = "hectares"
    """# Square centimetres <https://en.wikipedia.org/wiki/Square_centimetre> """  # noqa: E501
    SQUARE_CENTIMETRES = "square_centimetres"
    """# Square decimetres <https://en.wikipedia.org/wiki/Square_decimetre> """  # noqa: E501
    SQUARE_DECIMETRES = "square_decimetres"
    """# Square feet <https://en.wikipedia.org/wiki/Square_foot> """  # noqa: E501
    SQUARE_FEET = "square_feet"
    """# Square hectometres <https://en.wikipedia.org/wiki/Square_hectometre> """  # noqa: E501
    SQUARE_HECTOMETRES = "square_hectometres"
    """# Square inches <https://en.wikipedia.org/wiki/Square_inch> """  # noqa: E501
    SQUARE_INCHES = "square_inches"
    """# Square kilometres <https://en.wikipedia.org/wiki/Square_kilometre> """  # noqa: E501
    SQUARE_KILOMETRES = "square_kilometres"
    """# Square metres <https://en.wikipedia.org/wiki/Square_metre> """  # noqa: E501
    SQUARE_METRES = "square_metres"
    """# Square micrometres <https://en.wikipedia.org/wiki/Square_micrometre> """  # noqa: E501
    SQUARE_MICROMETRES = "square_micrometres"
    """# Square miles <https://en.wikipedia.org/wiki/Square_mile> """  # noqa: E501
    SQUARE_MILES = "square_miles"
    """# Square millimetres <https://en.wikipedia.org/wiki/Square_millimetre> """  # noqa: E501
    SQUARE_MILLIMETRES = "square_millimetres"
    """# Square nanometres <https://en.wikipedia.org/wiki/Square_nanometre> """  # noqa: E501
    SQUARE_NANOMETRES = "square_nanometres"
    """# Square yards <https://en.wikipedia.org/wiki/Square_mile> """  # noqa: E501
    SQUARE_YARDS = "square_yards"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class Currency(str, Enum):
    """Currency is the list of supported currencies.

    This comes from the Stripe API docs: For more details see <https://support.stripe.com/questions/which-currencies-does-stripe-support>."""  # noqa: E501

    """# United Arab Emirates Dirham """  # noqa: E501
    AED = "aed"
    """# Afghan Afghani """  # noqa: E501
    AFN = "afn"
    """# Albanian Lek """  # noqa: E501
    ALL = "all"
    """# Armenian Dram """  # noqa: E501
    AMD = "amd"
    """# Netherlands Antillean Gulden """  # noqa: E501
    ANG = "ang"
    """# Angolan Kwanza """  # noqa: E501
    AOA = "aoa"
    """# Argentine Peso """  # noqa: E501
    ARS = "ars"
    """# Australian Dollar """  # noqa: E501
    AUD = "aud"
    """# Aruban Florin """  # noqa: E501
    AWG = "awg"
    """# Azerbaijani Manat """  # noqa: E501
    AZN = "azn"
    """# Bosnia & Herzegovina Convertible Mark """  # noqa: E501
    BAM = "bam"
    """# Barbadian Dollar """  # noqa: E501
    BBD = "bbd"
    """# Bangladeshi Taka """  # noqa: E501
    BDT = "bdt"
    """# Bulgarian Lev """  # noqa: E501
    BGN = "bgn"
    """# Burundian Franc """  # noqa: E501
    BIF = "bif"
    """# Bermudian Dollar """  # noqa: E501
    BMD = "bmd"
    """# Brunei Dollar """  # noqa: E501
    BND = "bnd"
    """# Bolivian Boliviano """  # noqa: E501
    BOB = "bob"
    """# Brazilian Real """  # noqa: E501
    BRL = "brl"
    """# Bahamian Dollar """  # noqa: E501
    BSD = "bsd"
    """# Botswana Pula """  # noqa: E501
    BWP = "bwp"
    """# Belize Dollar """  # noqa: E501
    BZD = "bzd"
    """# Canadian Dollar """  # noqa: E501
    CAD = "cad"
    """# Congolese Franc """  # noqa: E501
    CDF = "cdf"
    """# Swiss Franc """  # noqa: E501
    CHF = "chf"
    """# Chilean Peso """  # noqa: E501
    CLP = "clp"
    """# Chinese Renminbi Yuan """  # noqa: E501
    CNY = "cny"
    """# Colombian Peso """  # noqa: E501
    COP = "cop"
    """# Costa Rican Colón """  # noqa: E501
    CRC = "crc"
    """# Cape Verdean Escudo """  # noqa: E501
    CVE = "cve"
    """# Czech Koruna """  # noqa: E501
    CZK = "czk"
    """# Djiboutian Franc """  # noqa: E501
    DJF = "djf"
    """# Danish Krone """  # noqa: E501
    DKK = "dkk"
    """# Dominican Peso """  # noqa: E501
    DOP = "dop"
    """# Algerian Dinar """  # noqa: E501
    DZD = "dzd"
    """# Estonian Kroon """  # noqa: E501
    EEK = "eek"
    """# Egyptian Pound """  # noqa: E501
    EGP = "egp"
    """# Ethiopian Birr """  # noqa: E501
    ETB = "etb"
    """# Euro """  # noqa: E501
    EUR = "eur"
    """# Fijian Dollar """  # noqa: E501
    FJD = "fjd"
    """# Falkland Islands Pound """  # noqa: E501
    FKP = "fkp"
    """# British Pound """  # noqa: E501
    GBP = "gbp"
    """# Georgian Lari """  # noqa: E501
    GEL = "gel"
    """# Gibraltar Pound """  # noqa: E501
    GIP = "gip"
    """# Gambian Dalasi """  # noqa: E501
    GMD = "gmd"
    """# Guinean Franc """  # noqa: E501
    GNF = "gnf"
    """# Guatemalan Quetzal """  # noqa: E501
    GTQ = "gtq"
    """# Guyanese Dollar """  # noqa: E501
    GYD = "gyd"
    """# Hong Kong Dollar """  # noqa: E501
    HKD = "hkd"
    """# Honduran Lempira """  # noqa: E501
    HNL = "hnl"
    """# Croatian Kuna """  # noqa: E501
    HRK = "hrk"
    """# Haitian Gourde """  # noqa: E501
    HTG = "htg"
    """# Hungarian Forint """  # noqa: E501
    HUF = "huf"
    """# Indonesian Rupiah """  # noqa: E501
    IDR = "idr"
    """# Israeli New Sheqel """  # noqa: E501
    ILS = "ils"
    """# Indian Rupee """  # noqa: E501
    INR = "inr"
    """# Icelandic Króna """  # noqa: E501
    ISK = "isk"
    """# Jamaican Dollar """  # noqa: E501
    JMD = "jmd"
    """# Japanese Yen """  # noqa: E501
    JPY = "jpy"
    """# Kenyan Shilling """  # noqa: E501
    KES = "kes"
    """# Kyrgyzstani Som """  # noqa: E501
    KGS = "kgs"
    """# Cambodian Riel """  # noqa: E501
    KHR = "khr"
    """# Comorian Franc """  # noqa: E501
    KMF = "kmf"
    """# South Korean Won """  # noqa: E501
    KRW = "krw"
    """# Cayman Islands Dollar """  # noqa: E501
    KYD = "kyd"
    """# Kazakhstani Tenge """  # noqa: E501
    KZT = "kzt"
    """# Lao Kip """  # noqa: E501
    LAK = "lak"
    """# Lebanese Pound """  # noqa: E501
    LBP = "lbp"
    """# Sri Lankan Rupee """  # noqa: E501
    LKR = "lkr"
    """# Liberian Dollar """  # noqa: E501
    LRD = "lrd"
    """# Lesotho Loti """  # noqa: E501
    LSL = "lsl"
    """# Lithuanian Litas """  # noqa: E501
    LTL = "ltl"
    """# Latvian Lats """  # noqa: E501
    LVL = "lvl"
    """# Moroccan Dirham """  # noqa: E501
    MAD = "mad"
    """# Moldovan Leu """  # noqa: E501
    MDL = "mdl"
    """# Malagasy Ariary """  # noqa: E501
    MGA = "mga"
    """# Macedonian Denar """  # noqa: E501
    MKD = "mkd"
    """# Mongolian Tögrög """  # noqa: E501
    MNT = "mnt"
    """# Macanese Pataca """  # noqa: E501
    MOP = "mop"
    """# Mauritanian Ouguiya """  # noqa: E501
    MRO = "mro"
    """# Mauritian Rupee """  # noqa: E501
    MUR = "mur"
    """# Maldivian Rufiyaa """  # noqa: E501
    MVR = "mvr"
    """# Malawian Kwacha """  # noqa: E501
    MWK = "mwk"
    """# Mexican Peso """  # noqa: E501
    MXN = "mxn"
    """# Malaysian Ringgit """  # noqa: E501
    MYR = "myr"
    """# Mozambican Metical """  # noqa: E501
    MZN = "mzn"
    """# Namibian Dollar """  # noqa: E501
    NAD = "nad"
    """# Nigerian Naira """  # noqa: E501
    NGN = "ngn"
    """# Nicaraguan Córdoba """  # noqa: E501
    NIO = "nio"
    """# Norwegian Krone """  # noqa: E501
    NOK = "nok"
    """# Nepalese Rupee """  # noqa: E501
    NPR = "npr"
    """# New Zealand Dollar """  # noqa: E501
    NZD = "nzd"
    """# Panamanian Balboa """  # noqa: E501
    PAB = "pab"
    """# Peruvian Nuevo Sol """  # noqa: E501
    PEN = "pen"
    """# Papua New Guinean Kina """  # noqa: E501
    PGK = "pgk"
    """# Philippine Peso """  # noqa: E501
    PHP = "php"
    """# Pakistani Rupee """  # noqa: E501
    PKR = "pkr"
    """# Polish Złoty """  # noqa: E501
    PLN = "pln"
    """# Paraguayan Guaraní """  # noqa: E501
    PYG = "pyg"
    """# Qatari Riyal """  # noqa: E501
    QAR = "qar"
    """# Romanian Leu """  # noqa: E501
    RON = "ron"
    """# Serbian Dinar """  # noqa: E501
    RSD = "rsd"
    """# Russian Ruble """  # noqa: E501
    RUB = "rub"
    """# Rwandan Franc """  # noqa: E501
    RWF = "rwf"
    """# Saudi Riyal """  # noqa: E501
    SAR = "sar"
    """# Solomon Islands Dollar """  # noqa: E501
    SBD = "sbd"
    """# Seychellois Rupee """  # noqa: E501
    SCR = "scr"
    """# Swedish Krona """  # noqa: E501
    SEK = "sek"
    """# Singapore Dollar """  # noqa: E501
    SGD = "sgd"
    """# Saint Helenian Pound """  # noqa: E501
    SHP = "shp"
    """# Sierra Leonean Leone """  # noqa: E501
    SLL = "sll"
    """# Somali Shilling """  # noqa: E501
    SOS = "sos"
    """# Surinamese Dollar """  # noqa: E501
    SRD = "srd"
    """# São Tomé and Príncipe Dobra """  # noqa: E501
    STD = "std"
    """# Salvadoran Colón """  # noqa: E501
    SVC = "svc"
    """# Swazi Lilangeni """  # noqa: E501
    SZL = "szl"
    """# Thai Baht """  # noqa: E501
    THB = "thb"
    """# Tajikistani Somoni """  # noqa: E501
    TJS = "tjs"
    """# Tongan Paʻanga """  # noqa: E501
    TOP = "top"
    """# Turkish Lira """  # noqa: E501
    TRY = "try"
    """# Trinidad and Tobago Dollar """  # noqa: E501
    TTD = "ttd"
    """# New Taiwan Dollar """  # noqa: E501
    TWD = "twd"
    """# Tanzanian Shilling """  # noqa: E501
    TZS = "tzs"
    """# Ukrainian Hryvnia """  # noqa: E501
    UAH = "uah"
    """# Ugandan Shilling """  # noqa: E501
    UGX = "ugx"
    """# United States Dollar """  # noqa: E501
    USD = "usd"
    """# Uruguayan Peso """  # noqa: E501
    UYU = "uyu"
    """# Uzbekistani Som """  # noqa: E501
    UZS = "uzs"
    """# Venezuelan Bolívar """  # noqa: E501
    VEF = "vef"
    """# Vietnamese Đồng """  # noqa: E501
    VND = "vnd"
    """# Vanuatu Vatu """  # noqa: E501
    VUV = "vuv"
    """# Samoan Tala """  # noqa: E501
    WST = "wst"
    """# Central African Cfa Franc """  # noqa: E501
    XAF = "xaf"
    """# East Caribbean Dollar """  # noqa: E501
    XCD = "xcd"
    """# West African Cfa Franc """  # noqa: E501
    XOF = "xof"
    """# Cfp Franc """  # noqa: E501
    XPF = "xpf"
    """# Yemeni Rial """  # noqa: E501
    YER = "yer"
    """# South African Rand """  # noqa: E501
    ZAR = "zar"
    """# Zambian Kwacha """  # noqa: E501
    ZMW = "zmw"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class PhysicsConstantName(str, Enum):
    """The valid types of phys constant names."""  # noqa: E501

    """# pi - Ratio of a circle's circumference to its diameter. <https://en.wikipedia.org/wiki/Pi> """  # noqa: E501
    PI = "pi"
    """# c - Speed of light in vacuum. <https://en.wikipedia.org/wiki//Speed_of_light> """  # noqa: E501
    C = "c"
    """# Speed of light in a vacuum. <https://en.wikipedia.org/wiki//Speed_of_light> """  # noqa: E501
    SPEED_OF_LIGHT = "speed_of_light"
    """# G - Newtonian constant of gravitation. <https://en.wikipedia.org/wiki/Gravitational_constant> """  # noqa: E501
    G = "G"
    """# Newtonian constant of gravitation. <https://en.wikipedia.org/wiki/Gravitational_constant> """  # noqa: E501
    NEWTONIAN_GRAVITATION = "newtonian_gravitation"
    """# h - Planck constant. <https://en.wikipedia.org/wiki/Planck_constant> """  # noqa: E501
    H = "h"
    """# Planck constant. <https://en.wikipedia.org/wiki/Planck_constant> """  # noqa: E501
    PLANCK_CONST = "planck_const"
    """# mu_0 - vacuum permeability. <https://en.wikipedia.org/wiki/Vacuum_permeability> """  # noqa: E501
    MU_0 = "mu_0"
    """# vacuum permeability. <https://en.wikipedia.org/wiki/Vacuum_permeability> """  # noqa: E501
    VACUUM_PERMEABILITY = "vacuum_permeability"
    """# ε_0 - vacuum permitivity. <https://en.wikipedia.org/wiki/Vacuum_permittivity> """  # noqa: E501
    E_0 = "E_0"
    """# vacuum permitivity. <https://en.wikipedia.org/wiki/Vacuum_permittivity>] """  # noqa: E501
    VACUUM_PERMITIVITY = "vacuum_permitivity"
    """# Z_0 - characteristic impedance of vacuum. <https://en.wikipedia.org/wiki/Impedance_of_free_space> """  # noqa: E501
    Z_0 = "Z_0"
    """# characteristic impedance of vacuum. <https://en.wikipedia.org/wiki/Impedance_of_free_space> """  # noqa: E501
    VACUUM_IMPEDANCE = "vacuum_impedance"
    """# k_e - Coulomb's constant. <https://en.wikipedia.org/wiki/Coulomb_constant> """  # noqa: E501
    K_E = "k_e"
    """# Coulomb's constant. <https://en.wikipedia.org/wiki/Coulomb_constant> """  # noqa: E501
    COULOMB_CONST = "coulomb_const"
    """# e - elementary charge. <https://en.wikipedia.org/wiki/Elementary_charge> """  # noqa: E501
    E = "e"
    """# elementary charge. <https://en.wikipedia.org/wiki/Elementary_charge> """  # noqa: E501
    ELEMENTARY_CHARGE = "elementary_charge"
    """# m_e - electron mass. <https://en.wikipedia.org/wiki/Electron_mass> """  # noqa: E501
    M_E = "m_e"
    """# electron mass. <https://en.wikipedia.org/wiki/Electron_mass> """  # noqa: E501
    ELECTRON_MASS = "electron_mass"
    """# m_p - proton mass. <https://en.wikipedia.org/wiki/Proton> """  # noqa: E501
    M_P = "m_p"
    """# proton mass. <https://en.wikipedia.org/wiki/Proton> """  # noqa: E501
    PROTON_MASS = "proton_mass"
    """# mu_B - Bohr magneton. <https://en.wikipedia.org/wiki/Bohr_magneton> """  # noqa: E501
    MU_B = "mu_B"
    """# Bohr magneton. <https://en.wikipedia.org/wiki/Bohr_magneton> """  # noqa: E501
    BOHR_MAGNETON = "bohr_magneton"
    """# NA - Avogadro's Number. <https://en.wikipedia.org/wiki/Avogadro_constant> """  # noqa: E501
    NA = "NA"
    """# Avogadro's Number. <https://en.wikipedia.org/wiki/Avogadro_constant> """  # noqa: E501
    AVOGADRO_NUM = "avogadro_num"
    """# R - Molar Gas constant. <https://en.wikipedia.org/wiki/Gas_constant> """  # noqa: E501
    R = "R"
    """# Molar Gas constant. <https://en.wikipedia.org/wiki/Gas_constant> """  # noqa: E501
    MOLAR_GAS_CONST = "molar_gas_const"
    """# K_B - Boltzmann constant. <https://en.wikipedia.org/wiki/Boltzmann_constant> """  # noqa: E501
    K_B = "K_B"
    """# Boltzmann constant. <https://en.wikipedia.org/wiki/Boltzmann_constant> """  # noqa: E501
    BOLTZMANN_CONST = "boltzmann_const"
    """# F - Faraday constant. <https://en.wikipedia.org/wiki/Faraday_constant> """  # noqa: E501
    F = "F"
    """# Faraday constant. <https://en.wikipedia.org/wiki/Faraday_constant> """  # noqa: E501
    FARADAY_CONST = "faraday_const"
    """# Sigma - Stefan-Boltzmann constant. <https://en.wikipedia.org/wiki/Stefan%E2%80%93Boltzmann_constant> """  # noqa: E501
    SIGMA = "sigma"
    """# Stefan-Boltzmann constant. <https://en.wikipedia.org/wiki/Stefan%E2%80%93Boltzmann_constant> """  # noqa: E501
    STEFAN_BOLTZMANN_CONST = "stefan_boltzmann_const"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class AiPluginAuthType(str, Enum):
    """AI plugin auth type."""  # noqa: E501

    """# None. """  # noqa: E501
    NONE = "none"
    """# User http. """  # noqa: E501
    USER_HTTP = "user_http"
    """# Service http. """  # noqa: E501
    SERVICE_HTTP = "service_http"
    """# OAuth. """  # noqa: E501
    OAUTH = "oauth"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.modeling_cmd_id import ModelingCmdId
from ..types import UNSET, Unset

S = TypeVar("S", bound="Extrude")


@attr.s(auto_attribs=True)
class Extrude:
    """Command for extruding a solid."""  # noqa: E501

    cap: Union[Unset, bool] = False
    distance: Union[Unset, float] = UNSET
    target: Union[Unset, ModelingCmdId] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        cap = self.cap
        distance = self.distance
        if not isinstance(self.target, Unset):
            target = self.target

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cap is not UNSET:
            field_dict["cap"] = cap
        if distance is not UNSET:
            field_dict["distance"] = distance
        if target is not UNSET:
            field_dict["target"] = target

        return field_dict

    @classmethod
    def from_dict(cls: Type[S], src_dict: Dict[str, Any]) -> S:
        d = src_dict.copy()
        cap = d.pop("cap", UNSET)

        distance = d.pop("distance", UNSET)

        _target = d.pop("target", UNSET)
        target: Union[Unset, ModelingCmdId]
        if isinstance(_target, Unset):
            target = UNSET
        else:
            target = ModelingCmdId(_target)

        extrude = cls(
            cap=cap,
            distance=distance,
            target=target,
        )

        extrude.additional_properties = d
        return extrude

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

H = TypeVar("H", bound="IndexInfo")


@attr.s(auto_attribs=True)
class IndexInfo:
    """IndexInfo contains information about a registry."""  # noqa: E501

    mirrors: Union[Unset, List[str]] = UNSET
    name: Union[Unset, str] = UNSET
    official: Union[Unset, bool] = False
    secure: Union[Unset, bool] = False

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        mirrors: Union[Unset, List[str]] = UNSET
        if not isinstance(self.mirrors, Unset):
            mirrors = self.mirrors
        name = self.name
        official = self.official
        secure = self.secure

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if mirrors is not UNSET:
            field_dict["mirrors"] = mirrors
        if name is not UNSET:
            field_dict["name"] = name
        if official is not UNSET:
            field_dict["official"] = official
        if secure is not UNSET:
            field_dict["secure"] = secure

        return field_dict

    @classmethod
    def from_dict(cls: Type[H], src_dict: Dict[str, Any]) -> H:
        d = src_dict.copy()
        mirrors = cast(List[str], d.pop("mirrors", UNSET))

        name = d.pop("name", UNSET)

        official = d.pop("official", UNSET)

        secure = d.pop("secure", UNSET)

        index_info = cls(
            mirrors=mirrors,
            name=name,
            official=official,
            secure=secure,
        )

        index_info.additional_properties = d
        return index_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.point2d import Point2d
from ..models.point3d import Point3d
from ..types import UNSET, Unset

F = TypeVar("F", bound="Line")


@attr.s(auto_attribs=True)
class Line:
    end: Union[Unset, Point3d] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.end, Unset):
            end = self.end

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if end is not UNSET:
            field_dict["end"] = end

        return field_dict

    @classmethod
    def from_dict(cls: Type[F], src_dict: Dict[str, Any]) -> F:
        d = src_dict.copy()
        _end = d.pop("end", UNSET)
        end: Union[Unset, Point3d]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = Point3d(_end)

        line = cls(
            end=end,
        )

        line.additional_properties = d
        return line

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


Z = TypeVar("Z", bound="Arc")


@attr.s(auto_attribs=True)
class Arc:
    angle_end: Union[Unset, float] = UNSET
    angle_start: Union[Unset, float] = UNSET
    center: Union[Unset, Point2d] = UNSET
    radius: Union[Unset, float] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        angle_end = self.angle_end
        angle_start = self.angle_start
        if not isinstance(self.center, Unset):
            center = self.center
        radius = self.radius

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if angle_end is not UNSET:
            field_dict["angle_end"] = angle_end
        if angle_start is not UNSET:
            field_dict["angle_start"] = angle_start
        if center is not UNSET:
            field_dict["center"] = center
        if radius is not UNSET:
            field_dict["radius"] = radius

        return field_dict

    @classmethod
    def from_dict(cls: Type[Z], src_dict: Dict[str, Any]) -> Z:
        d = src_dict.copy()
        angle_end = d.pop("angle_end", UNSET)

        angle_start = d.pop("angle_start", UNSET)

        _center = d.pop("center", UNSET)
        center: Union[Unset, Point2d]
        if isinstance(_center, Unset):
            center = UNSET
        else:
            center = Point2d(_center)

        radius = d.pop("radius", UNSET)

        arc = cls(
            angle_end=angle_end,
            angle_start=angle_start,
            center=center,
            radius=radius,
        )

        arc.additional_properties = d
        return arc

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


G = TypeVar("G", bound="Bezier")


@attr.s(auto_attribs=True)
class Bezier:
    control1: Union[Unset, Point3d] = UNSET
    control2: Union[Unset, Point3d] = UNSET
    end: Union[Unset, Point3d] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.control1, Unset):
            control1 = self.control1
        if not isinstance(self.control2, Unset):
            control2 = self.control2
        if not isinstance(self.end, Unset):
            end = self.end

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if control1 is not UNSET:
            field_dict["control1"] = control1
        if control2 is not UNSET:
            field_dict["control2"] = control2
        if end is not UNSET:
            field_dict["end"] = end

        return field_dict

    @classmethod
    def from_dict(cls: Type[G], src_dict: Dict[str, Any]) -> G:
        d = src_dict.copy()
        _control1 = d.pop("control1", UNSET)
        control1: Union[Unset, Point3d]
        if isinstance(_control1, Unset):
            control1 = UNSET
        else:
            control1 = Point3d(_control1)

        _control2 = d.pop("control2", UNSET)
        control2: Union[Unset, Point3d]
        if isinstance(_control2, Unset):
            control2 = UNSET
        else:
            control2 = Point3d(_control2)

        _end = d.pop("end", UNSET)
        end: Union[Unset, Point3d]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = Point3d(_end)

        bezier = cls(
            control1=control1,
            control2=control2,
            end=end,
        )

        bezier.additional_properties = d
        return bezier

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties


PathSegment = Union[Line, Arc, Bezier]



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.ai_plugin_auth_type import AiPluginAuthType
from ..models.ai_plugin_http_auth_type import AiPluginHttpAuthType
from ..types import UNSET, Unset

B = TypeVar("B", bound="AiPluginAuth")


@attr.s(auto_attribs=True)
class AiPluginAuth:
    """AI plugin auth information."""  # noqa: E501

    authorization_type: Union[Unset, AiPluginHttpAuthType] = UNSET
    type: Union[Unset, AiPluginAuthType] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.authorization_type, Unset):
            authorization_type = self.authorization_type
        if not isinstance(self.type, Unset):
            type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if authorization_type is not UNSET:
            field_dict["authorization_type"] = authorization_type
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[B], src_dict: Dict[str, Any]) -> B:
        d = src_dict.copy()
        _authorization_type = d.pop("authorization_type", UNSET)
        authorization_type: Union[Unset, AiPluginHttpAuthType]
        if isinstance(_authorization_type, Unset):
            authorization_type = UNSET
        else:
            authorization_type = _authorization_type  # type: ignore[arg-type]

        _type = d.pop("type", UNSET)
        type: Union[Unset, AiPluginAuthType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = _type  # type: ignore[arg-type]

        ai_plugin_auth = cls(
            authorization_type=authorization_type,
            type=type,
        )

        ai_plugin_auth.additional_properties = d
        return ai_plugin_auth

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.method import Method
from ..models.uuid import Uuid
from ..types import UNSET, Unset

S = TypeVar("S", bound="ApiCallWithPrice")


@attr.s(auto_attribs=True)
class ApiCallWithPrice:
    """An API call with the price.

    This is a join of the `ApiCall` and `ApiCallPrice` tables."""  # noqa: E501

    completed_at: Union[Unset, datetime.datetime] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    duration: Union[Unset, int] = UNSET
    email: Union[Unset, str] = UNSET
    endpoint: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    ip_address: Union[Unset, str] = UNSET
    litterbox: Union[Unset, bool] = False
    method: Union[Unset, Method] = UNSET
    minutes: Union[Unset, int] = UNSET
    origin: Union[Unset, str] = UNSET
    price: Union[Unset, float] = UNSET
    request_body: Union[Unset, str] = UNSET
    request_query_params: Union[Unset, str] = UNSET
    response_body: Union[Unset, str] = UNSET
    started_at: Union[Unset, datetime.datetime] = UNSET
    status_code: Union[Unset, int] = UNSET
    stripe_invoice_item_id: Union[Unset, str] = UNSET
    token: Union[Unset, str] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    user_agent: Union[Unset, str] = UNSET
    user_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        completed_at: Union[Unset, str] = UNSET
        if not isinstance(self.completed_at, Unset):
            completed_at = self.completed_at.isoformat()
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        duration = self.duration
        email = self.email
        endpoint = self.endpoint
        id = self.id
        ip_address = self.ip_address
        litterbox = self.litterbox
        if not isinstance(self.method, Unset):
            method = self.method
        minutes = self.minutes
        origin = self.origin
        price = self.price
        request_body = self.request_body
        request_query_params = self.request_query_params
        response_body = self.response_body
        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()
        status_code = self.status_code
        stripe_invoice_item_id = self.stripe_invoice_item_id
        token = self.token
        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()
        user_agent = self.user_agent
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if duration is not UNSET:
            field_dict["duration"] = duration
        if email is not UNSET:
            field_dict["email"] = email
        if endpoint is not UNSET:
            field_dict["endpoint"] = endpoint
        if id is not UNSET:
            field_dict["id"] = id
        if ip_address is not UNSET:
            field_dict["ip_address"] = ip_address
        if litterbox is not UNSET:
            field_dict["litterbox"] = litterbox
        if method is not UNSET:
            field_dict["method"] = method
        if minutes is not UNSET:
            field_dict["minutes"] = minutes
        if origin is not UNSET:
            field_dict["origin"] = origin
        if price is not UNSET:
            field_dict["price"] = price
        if request_body is not UNSET:
            field_dict["request_body"] = request_body
        if request_query_params is not UNSET:
            field_dict["request_query_params"] = request_query_params
        if response_body is not UNSET:
            field_dict["response_body"] = response_body
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if status_code is not UNSET:
            field_dict["status_code"] = status_code
        if stripe_invoice_item_id is not UNSET:
            field_dict["stripe_invoice_item_id"] = stripe_invoice_item_id
        if token is not UNSET:
            field_dict["token"] = token
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if user_agent is not UNSET:
            field_dict["user_agent"] = user_agent
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[S], src_dict: Dict[str, Any]) -> S:
        d = src_dict.copy()
        _completed_at = d.pop("completed_at", UNSET)
        completed_at: Union[Unset, datetime.datetime]
        if isinstance(_completed_at, Unset):
            completed_at = UNSET
        else:
            completed_at = isoparse(_completed_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        duration = d.pop("duration", UNSET)

        email = d.pop("email", UNSET)

        endpoint = d.pop("endpoint", UNSET)

        _id = d.pop("id", UNSET)
        id: Union[Unset, Uuid]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = Uuid(_id)

        ip_address = d.pop("ip_address", UNSET)

        litterbox = d.pop("litterbox", UNSET)

        _method = d.pop("method", UNSET)
        method: Union[Unset, Method]
        if isinstance(_method, Unset):
            method = UNSET
        else:
            method = _method  # type: ignore[arg-type]

        minutes = d.pop("minutes", UNSET)

        origin = d.pop("origin", UNSET)

        price = d.pop("price", UNSET)

        request_body = d.pop("request_body", UNSET)

        request_query_params = d.pop("request_query_params", UNSET)

        response_body = d.pop("response_body", UNSET)

        _started_at = d.pop("started_at", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        status_code = d.pop("status_code", UNSET)

        stripe_invoice_item_id = d.pop("stripe_invoice_item_id", UNSET)

        _token = d.pop("token", UNSET)
        token: Union[Unset, Uuid]
        if isinstance(_token, Unset):
            token = UNSET
        else:
            token = Uuid(_token)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        user_agent = d.pop("user_agent", UNSET)

        user_id = d.pop("user_id", UNSET)

        api_call_with_price = cls(
            completed_at=completed_at,
            created_at=created_at,
            duration=duration,
            email=email,
            endpoint=endpoint,
            id=id,
            ip_address=ip_address,
            litterbox=litterbox,
            method=method,
            minutes=minutes,
            origin=origin,
            price=price,
            request_body=request_body,
            request_query_params=request_query_params,
            response_body=response_body,
            started_at=started_at,
            status_code=status_code,
            stripe_invoice_item_id=stripe_invoice_item_id,
            token=token,
            updated_at=updated_at,
            user_agent=user_agent,
            user_id=user_id,
        )

        api_call_with_price.additional_properties = d
        return api_call_with_price

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

P = TypeVar("P", bound="ApiCallQueryGroup")


@attr.s(auto_attribs=True)
class ApiCallQueryGroup:
    """A response for a query on the API call table that is grouped by something."""  # noqa: E501

    count: Union[Unset, int] = UNSET
    query: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        count = self.count
        query = self.query

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if count is not UNSET:
            field_dict["count"] = count
        if query is not UNSET:
            field_dict["query"] = query

        return field_dict

    @classmethod
    def from_dict(cls: Type[P], src_dict: Dict[str, Any]) -> P:
        d = src_dict.copy()
        count = d.pop("count", UNSET)

        query = d.pop("query", UNSET)

        api_call_query_group = cls(
            count=count,
            query=query,
        )

        api_call_query_group.additional_properties = d
        return api_call_query_group

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.currency import Currency
from ..models.new_address import NewAddress
from ..types import UNSET, Unset

J = TypeVar("J", bound="Customer")


@attr.s(auto_attribs=True)
class Customer:
    """The resource representing a payment "Customer"."""  # noqa: E501

    address: Union[Unset, NewAddress] = UNSET
    balance: Union[Unset, float] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    currency: Union[Unset, Currency] = UNSET
    delinquent: Union[Unset, bool] = False
    email: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    metadata: Union[Unset, Any] = UNSET
    name: Union[Unset, str] = UNSET
    phone: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        if not isinstance(self.address, Unset):
            address = self.address
        balance = self.balance
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()
        if not isinstance(self.currency, Unset):
            currency = self.currency
        delinquent = self.delinquent
        email = self.email
        id = self.id
        metadata = self.metadata
        name = self.name
        phone = self.phone

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if address is not UNSET:
            field_dict["address"] = address
        if balance is not UNSET:
            field_dict["balance"] = balance
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if currency is not UNSET:
            field_dict["currency"] = currency
        if delinquent is not UNSET:
            field_dict["delinquent"] = delinquent
        if email is not UNSET:
            field_dict["email"] = email
        if id is not UNSET:
            field_dict["id"] = id
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if name is not UNSET:
            field_dict["name"] = name
        if phone is not UNSET:
            field_dict["phone"] = phone

        return field_dict

    @classmethod
    def from_dict(cls: Type[J], src_dict: Dict[str, Any]) -> J:
        d = src_dict.copy()
        _address = d.pop("address", UNSET)
        address: Union[Unset, NewAddress]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = NewAddress(_address)

        balance = d.pop("balance", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _currency = d.pop("currency", UNSET)
        currency: Union[Unset, Currency]
        if isinstance(_currency, Unset):
            currency = UNSET
        else:
            currency = _currency  # type: ignore[arg-type]

        delinquent = d.pop("delinquent", UNSET)

        email = d.pop("email", UNSET)

        id = d.pop("id", UNSET)

        metadata = d.pop("metadata", UNSET)
        name = d.pop("name", UNSET)

        phone = d.pop("phone", UNSET)

        customer = cls(
            address=address,
            balance=balance,
            created_at=created_at,
            currency=currency,
            delinquent=delinquent,
            email=email,
            id=id,
            metadata=metadata,
            name=name,
            phone=phone,
        )

        customer.additional_properties = d
        return customer

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

E = TypeVar("E", bound="Point2d")


@attr.s(auto_attribs=True)
class Point2d:
    """A point in 2D space"""  # noqa: E501

    x: Union[Unset, float] = UNSET
    y: Union[Unset, float] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        x = self.x
        y = self.y

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if x is not UNSET:
            field_dict["x"] = x
        if y is not UNSET:
            field_dict["y"] = y

        return field_dict

    @classmethod
    def from_dict(cls: Type[E], src_dict: Dict[str, Any]) -> E:
        d = src_dict.copy()
        x = d.pop("x", UNSET)

        y = d.pop("y", UNSET)

        point2d = cls(
            x=x,
            y=y,
        )

        point2d.additional_properties = d
        return point2d

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

V = TypeVar("V", bound="DeviceAuthRequestForm")


@attr.s(auto_attribs=True)
class DeviceAuthRequestForm:
    """The request parameters for the OAuth 2.0 Device Authorization Grant flow."""  # noqa: E501

    client_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        client_id = self.client_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if client_id is not UNSET:
            field_dict["client_id"] = client_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[V], src_dict: Dict[str, Any]) -> V:
        d = src_dict.copy()
        client_id = d.pop("client_id", UNSET)

        device_auth_request_form = cls(
            client_id=client_id,
        )

        device_auth_request_form.additional_properties = d
        return device_auth_request_form

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

R = TypeVar("R", bound="ModelingCmdReqBatch")


@attr.s(auto_attribs=True)
class ModelingCmdReqBatch:
    """A batch set of graphics commands submitted to the KittyCAD engine via the Modeling API."""  # noqa: E501

    cmds: Union[Unset, Any] = UNSET
    file_id: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        cmds = self.cmds
        file_id = self.file_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cmds is not UNSET:
            field_dict["cmds"] = cmds
        if file_id is not UNSET:
            field_dict["file_id"] = file_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[R], src_dict: Dict[str, Any]) -> R:
        d = src_dict.copy()
        cmds = d.pop("cmds", UNSET)
        file_id = d.pop("file_id", UNSET)

        modeling_cmd_req_batch = cls(
            cmds=cmds,
            file_id=file_id,
        )

        modeling_cmd_req_batch.additional_properties = d
        return modeling_cmd_req_batch

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class UnitTemperature(str, Enum):
    """The valid types of temperature units."""  # noqa: E501

    """# Celsius <https://en.wikipedia.org/wiki/Celsius> """  # noqa: E501
    CELSIUS = "celsius"
    """# Fahrenheit <https://en.wikipedia.org/wiki/Fahrenheit> """  # noqa: E501
    FAHRENHEIT = "fahrenheit"
    """# Kelvin <https://en.wikipedia.org/wiki/Kelvin> """  # noqa: E501
    KELVIN = "kelvin"
    """# Rankine <https://en.wikipedia.org/wiki/Rankine_scale> """  # noqa: E501
    RANKINE = "rankine"

    def __str__(self) -> str:
        return str(self.value)
